/*=====================================================================
  File:      AppBase.cs
 
  Summary:   This is a base class used by the various samples for
             various configuration and commandline parsing needs.
 
---------------------------------------------------------------------
  This file is part of the Web Services Enhancements 3.0 for Microsoft .NET Samples.

  Copyright (C) Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

This sample is designed to demonstrate WSE features and is not intended 
for production use. Code and policy for a production application must be 
developed to meet the specific data and security requirements of the 
application.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/

using System;
using System.Collections;
using System.Configuration;
using System.Text;
using System.Net;
using System.IO;
using System.Web.Services.Protocols;
using System.Security.Cryptography.X509Certificates;

using Microsoft.Web.Services3.Configuration;
using Microsoft.Web.Services3.Security;
using Microsoft.Web.Services3.Security.Tokens;
using Microsoft.Web.Services3.Security.X509;

namespace Microsoft.Web.Services3.QuickStart
{
    /// <summary>
    /// This is a base class used by the various samples for
    /// various configuration and commandline parsing needs.
    /// </summary>
    public class AppBase
    {
        SoapProtocolVersion _soapVersion = SoapProtocolVersion.Default;
        public static string SubjectKeyIdentifierOID = "2.5.29.14";

        protected SoapProtocolVersion SoapVersion
        {
            get
            {
                return _soapVersion;
            }
        }

        public AppBase()
        {
            string soapVersion = System.Configuration.ConfigurationManager.AppSettings["soapversion"];

            switch (soapVersion)
            {
                case "soap11":
                    _soapVersion = SoapProtocolVersion.Soap11;
                    break;
                case "soap12":
                    _soapVersion = SoapProtocolVersion.Soap12;
                    break;
                default:
                case "default":
                    _soapVersion = SoapProtocolVersion.Default;
                    break;
            }
        }

        // Replace the key id below with your own certificate Id in Base64String here
        // You can get these values for any cert with the WSE 3.0 Certificate Tool
        public static string ClientBase64KeyIdUseCAPI = "gBfo0147lM6cKnTbbMSuMVvmFY4=";
        public static string ServerBase64KeyIdUseCAPI = "bBwPfItvKp3b6TNDq+14qs58VJQ=";

        // If users select rfc3280 flag in the <x509> configuration,
        // they need to use the following certificate Id to verify the incoming certificate
        public static string ClientBase64KeyIdUseRFC3280 = "tyOZ9U6iyFdA0PwC6qgRfePd57s=";
        public static string ServerBase64KeyIdUseRFC3280 = "M9WWSXczVFt2otJ3adPjfmbUxU8=";

         // If users select sha1 flag in the <x509> configuration,
        // they need to use the following certificate Id to verify the incoming certificate
        public static string ClientBase64KeyId = "ynYBOBtFeFArYriAmCVmTx5436I=";
        public static string ServerBase64KeyId = "h9ksyrcUww2w4LrmubC2W11t988=";

        /// <summary>
        /// Looks for any configuration settings that may modify
        /// the Web service proxy used and modifies the proxy accordingly.
        /// </summary>
        protected void ConfigureProxy(SoapHttpClientProtocol protocol)
        {
            string remoteHost = ConfigurationManager.AppSettings["remoteHost"];
            if (remoteHost != null)
            {
                Uri remoteHostUri = new Uri(remoteHost);
                Uri protocolUrl = new Uri(protocol.Url);
                Uri newUri = new Uri(remoteHostUri, protocolUrl.AbsolutePath);

                if (protocol is Microsoft.Web.Services3.WebServicesClientProtocol) 
                {
                    ((Microsoft.Web.Services3.WebServicesClientProtocol)protocol).Url = newUri.AbsoluteUri;
                }
                else
                {
                    protocol.Url = newUri.AbsoluteUri;
                }
            }

            protocol.SoapVersion = SoapVersion;
        }
    
        protected static void Error(Exception e) 
        {
            Console.WriteLine("****** Exception Raised ******");

            if (e is ResponseProcessingException)
                Console.WriteLine(((ResponseProcessingException)e).Response.OuterXml);

            StringBuilder sb = new StringBuilder();
            ProcessException(e, sb);
            Console.WriteLine(sb.ToString());
            Console.WriteLine("******************************");
        }

        private static void ProcessException(Exception e, StringBuilder sb)
        {
            if (e != null)
            {
                if (e is WebException)
                {
                    // Process the Web Exception.
                    WebException webExcep = e as WebException;
                    if (webExcep.Response != null)
                    {
                        WebResponse response = webExcep.Response;
                        string str = webExcep.Message;
                        Stream responseStream = response.GetResponseStream();

                        if ( responseStream.CanRead ) 
                        {
                            StreamReader reader = new StreamReader(responseStream, System.Text.Encoding.UTF8);
                            string excepStr = reader.ReadToEnd();
                            sb.Append("Web Exception Occured: " + excepStr);
                        }
                        else 
                        {
                            sb.Append("Web Exception Occured: " + e.ToString());
                        }
                    }
                    else
                    {
                        sb.Append("Web Exception Occured: " + e.ToString());
                    }
                }
                else 
                {
                    if (e is System.Web.Services.Protocols.SoapException)
                    {
                        System.Web.Services.Protocols.SoapException se = e as System.Web.Services.Protocols.SoapException;
                        sb.Append("System.Web.Services.Protocols.SoapException:");
                        sb.Append(Environment.NewLine);
                        sb.Append("SOAP-Fault code: " + se.Code.ToString());
                        sb.Append(Environment.NewLine);
                        sb.Append("Message: ");
                    }
                    else
                    {
                        sb.Append(e.GetType().FullName);
                        sb.Append(": ");
                    }

                    sb.Append(e.Message);

                    if (e.InnerException != null)
                    {
                        sb.Append(" ---> ");
                        ProcessException(e.InnerException, sb);
                        sb.Append(Environment.NewLine);
                        sb.Append("--- End of Inner Exception ---");
                    }

                    if (e.StackTrace != null)
                    {
                        sb.Append(Environment.NewLine);
                        sb.Append(e.StackTrace);
                    }
                }
            }
        }

        /// <summary>
        /// Routine to compare two byte arrays
        /// </summary>
        /// <param name="a">First Byte Array</param>
        /// <param name="b">Second Byte Array</param>
        /// <returns>true or false</returns>
        public static bool CompareArray(byte[] a, byte[] b)
        {
            if (a != null && b != null && a.Length == b.Length)
            {
                int index = a.Length;
                while (--index > -1)
                    if (a[index] != b[index])
                        return false;
                return true;
            }
            else if (a == null && b == null)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Return the client certificate usually used for signing the message
        /// </summary>
        /// <param name="selectFromList">a flag to decide if we need to use the X.509 dialog</param>
        /// <returns>X509Security found for client</returns>
        public static X509SecurityToken GetClientToken(bool selectFromList)
        {
            X509SecurityToken token = null;

            // Open the CurrentUser Certificate Store and try MyStore only
            if ( selectFromList )
            {
                token = RetrieveTokenFromDialog("My", StoreLocation.CurrentUser);
            }
            else 
            {
                token = RetrieveTokenFromStore("My", StoreLocation.CurrentUser, ClientBase64KeyId);
            }

            return token;
        }

        /// <summary>
        /// Return the server certificate usually used for encrypting the message.
        /// </summary>
        /// <param name="selectFromList">a flag to decide if we need to use the X.509 dialog</param>
        /// <returns>X509Security found for server</returns>
        public static X509SecurityToken GetServerToken(bool selectFromList, bool isClient)
        {
            X509SecurityToken token = null;
            string storeName = null;
            StoreLocation storeLocation;

            if (isClient)
            {
                // Open the CurrentUser Certificate Store and try OtherPeople first
                storeName = "AddressBook";
                storeLocation = StoreLocation.CurrentUser;
            }
            else
            {
                // For server, open the LocalMachine Certificate Store and try Personal store.
                storeName = "My";
                storeLocation = StoreLocation.LocalMachine;
            }

            if ( selectFromList )
            {
               token = RetrieveTokenFromDialog(storeName, storeLocation);

                if ( token == null )
                {
                    if (isClient)
                    {
                        token = RetrieveTokenFromDialog("My", StoreLocation.CurrentUser);
                    }
                    else
                    {
                        // For server the server Certificate should be installed in the Personal 
                        // store. We tried that store and either we were unable to open the store
                        // or the certificate does not exist. So we don't have a fallback for this
                        // case.
                    }
                }
            }
            else 
            {
                token = RetrieveTokenFromStore(storeName, storeLocation, ServerBase64KeyId);
           
                //
                // If we failed to retrieve it from the OtherPeople,
                // we now try the MyStore
                //
                if ( token == null )
                {
                    if (isClient)
                    {
                        token = RetrieveTokenFromStore("My", StoreLocation.CurrentUser, ServerBase64KeyId);
                    }
                    else
                    {
                        // For server the server Certificate should be installed in the Personal 
                        // store. We tried that store and either we were unable to open the store
                        // or the certificate does not exist. So we don't have a fallback for this
                        // case.
                    }
                }
            }

            return token;
        }

        private static X509SecurityToken RetrieveTokenFromDialog(string storeName, StoreLocation storeLocation) 
        {
            if (storeName == null)
                throw new ArgumentNullException("storeName");

            X509Store store = new X509Store(storeName, storeLocation);

            X509SecurityToken token = null;

            try 
            {
                store.Open(OpenFlags.OpenExistingOnly | OpenFlags.ReadOnly);
                X509Certificate2Collection selectedCerts = X509Certificate2UI.SelectFromCollection(store.Certificates, "Select Certificate", "Choose a Certificate below.", X509SelectionFlag.SingleSelection);
                if (selectedCerts.Count != 0)
                    token = new X509SecurityToken(selectedCerts[0]);
            }
            finally 
            {
                if ( store != null )
                    store.Close();
            }

            return token;
        }

        private static X509SecurityToken RetrieveTokenFromStore(string storeName, StoreLocation storeLocation, string keyId) 
        {
            if (storeName == null)
                throw new ArgumentNullException("storeName");

            X509SecurityToken token = null;
            byte[] keyIdentifier = Convert.FromBase64String(keyId);

            X509Certificate2Collection certs = X509Util.FindCertificateByKeyIdentifier(keyIdentifier, storeLocation, storeName);
            if (certs.Count > 0)
            {
                // Get the first certificate in the collection
                token = new X509SecurityToken( certs[0] );
            }        

            return token;
        }

    }
}
