/*=====================================================================
  File:      PolicyAssetionHelper.cs
 
  Summary:   The following methods are helper methods that can be used inside 
             of the ReceiveSecurityFilter and the SendSecurityFilter filter classes.
             Specifically they can be used with the overrides of the methods that return
             a Security context for the pipeline. These methods
  
             public override void ValidateMessageSecurity(SoapEnvelope envelope, Security security)
             public override void SecureMessage(SoapEnvelope envelope, Security security)
 
             See the Quickstart Advanced samples for examples of Assertion filters.
             They are provided here to help migraton from WSE 2.0 code samples although
             generally unless you are doing some special security processing in an Assertion
             they are not needed.
 
---------------------------------------------------------------------
  This file is part of the Web Services Enhancements 3.0 for Microsoft .NET Samples.

  Copyright (C) Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

This sample is designed to demonstrate WSE features and is not intended 
for production use. Code and policy for a production application must be 
developed to meet the specific data and security requirements of the 
application.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/

using System;
using System.Collections.Generic;
using System.Configuration;
using System.Text;

using Microsoft.Web.Services3.Security;
using Microsoft.Web.Services3.Security.Tokens;

namespace Microsoft.Web.Services3.QuickStart
{
    public class PolicySecurityFilterHelper
    {
        /// <summary>
        /// This method checks if the incoming message has signed all the important message parts
        /// such as soap:Body, all the addressing headers and timestamp.  
        /// </summary>
        /// <param name="context"></param>
        /// <returns>The signing token</returns>
        public static SecurityToken GetSigningToken(SoapEnvelope envelope, Microsoft.Web.Services3.Security.Security security)
        {
            foreach (ISecurityElement element in security.Elements)
            {
                if (element is MessageSignature)
                {
                    // The given context contains a Signature element.
                    MessageSignature sig = element as MessageSignature;

                    if (CheckSignature(envelope, security, sig))
                    {
                        // The SOAP Body is signed.
                        return sig.SigningToken;
                    }
                }
            }
            return null;
        }

        static bool CheckSignature(SoapEnvelope envelope, Microsoft.Web.Services3.Security.Security security, MessageSignature signature)
        {
            //
            // Now verify which parts of the message were actually signed.
            //
            SoapContext context = envelope.Context;
            SignatureOptions actualOptions = signature.SignatureOptions;
            SignatureOptions expectedOptions = SignatureOptions.IncludeSoapBody;

            if (security != null && security.Timestamp != null)
                expectedOptions |= SignatureOptions.IncludeTimestamp;

            //
            // The <Action> and <To> are required addressing elements.
            //
            expectedOptions |= SignatureOptions.IncludeAction;
            expectedOptions |= SignatureOptions.IncludeTo;

            if (context.Addressing.FaultTo != null && context.Addressing.FaultTo.TargetElement != null)
                expectedOptions |= SignatureOptions.IncludeFaultTo;

            if (context.Addressing.From != null && context.Addressing.From.TargetElement != null)
                expectedOptions |= SignatureOptions.IncludeFrom;

            if (context.Addressing.MessageID != null && context.Addressing.MessageID.TargetElement != null)
                expectedOptions |= SignatureOptions.IncludeMessageId;

            if (context.Addressing.RelatesTo != null && context.Addressing.RelatesTo.TargetElement != null)
                expectedOptions |= SignatureOptions.IncludeRelatesTo;

            if (context.Addressing.ReplyTo != null && context.Addressing.ReplyTo.TargetElement != null)
                expectedOptions |= SignatureOptions.IncludeReplyTo;
            //
            // Check if the all the expected options are the present.
            //
            return ((expectedOptions & actualOptions) == expectedOptions);

        }

        /// <summary>
        /// This method checks if the incoming message has encrypted the soap message body 
        /// </summary>
        /// <param name="context">the soap context to search for</param>
        /// <returns>The encrypting token</returns>
        public static SecurityToken GetEncryptingToken(Microsoft.Web.Services3.Security.Security security)
        {
            SecurityToken encryptingToken = null;

            foreach (ISecurityElement element in security.Elements)
            {
                if (element is EncryptedData)
                {
                    EncryptedData encryptedData = element as EncryptedData;
                    System.Xml.XmlElement targetElement = encryptedData.TargetElement;

                    if (SoapHelper.IsBodyElement(targetElement))
                    {
                        // The given context has the Body element Encrypted.
                        encryptingToken = encryptedData.SecurityToken;
                    }
                }
            }

            return encryptingToken;
        }
    }
}
